/*
** Module   : ServerState.java
** Abstract : stores server state vars that must be sync'd with the client
**
** Copyright (c) 2006-2011, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 GES 20060726   @28202 Created initial version to contain a simple
**                           set of server-side state.
** 002 GES 20060728   @28220 Added data and constructors (copy and 
**                           default).
** 003 GES 20060731   @28241 Frame hiding/pausing and destroy are now
**                           deferred.
** 004 GES 20060801   @28299 Converted to Externalizable to improve
**                           efficiency because Serializable has unneeded
**                           processing (for versioning...) that is
**                           executed on "inflation".
** 005 SIY 20060804   @28455 Unified storage for all operations.
** 006 SIY 20080502   @38194 Inferred generic types.
** 007 SIY 20090326   @41711 Added external procedure call nesting counter.
** 008 SIY 20090701   @43024 Moved instance clearing from LogicalTerminal.
**                           Added formatCheckingId variable.
** 009 SIY 20110112          Added color mapping transfer from server to client.
** 010 HC  20140119          Implemented runtime support for the COLOR 
**                           protermcap-attribute.
*/

package com.goldencode.p2j.ui;

import java.io.*;
import java.util.*;

/**
 * This class is a container for server-side state which can be easily 
 * transported to or from the client.
 */
public class ServerState
implements Externalizable
{
   /** Tracks the number of I/O interactions within the current block. */
   public int interactions = 0;
   
   /** Tracks any conditional up or down that must be applied. */
   public int conditionalUpDown = 0;
   
   /** Tracks the frame ID associated with the conditional up or down. */
   public int conditionalFrameId = 0;
   
   /** List of deferred hiding/pausing state changes. */
   public List<MarkEntry> markList = null;

   /** ID of frame which should enable hard format check. */
   public int formatCheckId = 0;
   
   /** Current procedure nesting. */
   public int nesting = 0;

   /**
    * Default constructor.
    */
   public ServerState()
   {
   }
   
   /**
    * Copy constructor.
    * 
    * @param   old
    *          The instance to duplicate.
    */
   public ServerState(ServerState old)
   {
      this.interactions       = old.interactions;
      this.conditionalUpDown  = old.conditionalUpDown;
      this.conditionalFrameId = old.conditionalFrameId;
      this.markList           = old.markList;
      this.formatCheckId      = old.formatCheckId;
   }

   /**
    * Reset state.  
    */
   public void reset()
   {
      // reset the conditional stuff as this is set externally as a
      // "one-shot" thing
      conditionalUpDown  = 0;
      conditionalFrameId = 0;
      
      // clear the hide/pause/destroy lists since they are one-time only
      markList = null;
      
      // reset format check frame ID
      formatCheckId = 0;

      // note that lastColorIndex is not reset intentionally
   }
   
   /**
    * Replacement for the default object reading method.
    * 
    * @param   in
    *          Input source from which fields will be restored.
    * 
    * @throws  IOException
    *          In case of I/O errors.
    * @throws  ClassNotFoundException
    *          If payload can't be instantiated.
    */
   @Override
   public void readExternal(ObjectInput in)
   throws IOException,
          ClassNotFoundException
   {
      interactions       = in.readInt();
      conditionalUpDown  = in.readInt();
      conditionalFrameId = in.readInt();
      nesting            = in.readInt();
      formatCheckId      = in.readInt();

      extractMarkEntries((MarkEntry[]) in.readObject());
   }

   /**
    * Replacement for the default object writing method.
    * 
    * @param   out
    *          The output destination to which fields will be saved.
    * 
    * @throws  IOException
    *          In case of I/O errors.
    */
   @Override
   public void writeExternal(ObjectOutput out)
   throws IOException
   {
      out.writeInt(interactions);
      out.writeInt(conditionalUpDown);
      out.writeInt(conditionalFrameId);
      out.writeInt(nesting);
      out.writeInt(formatCheckId);

      out.writeObject(markListToArray());
   }

   /**
    * Convert array of mark entries to list.
    *
    * @param   markArray
    *          Array of mark entries.
    */
   private void extractMarkEntries(MarkEntry[] markArray)
   {
      if (markArray.length > 0)
      {
         markList = new ArrayList<MarkEntry>(Arrays.asList(markArray));
      }
   }

   /**
    * Convert mark entries to array.
    *
    * @return  array of mark entries.
    */
   private MarkEntry[] markListToArray()
   {
      MarkEntry[] marray;

      if (markList == null)
         marray = new MarkEntry[0];
      else
         marray = markList.toArray(new MarkEntry[markList.size()]);

      return marray;
   }
}
